

/* 
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Patrick Boenzli
   co-programmer: ...
*/


#include "track_manager.h"


using namespace std;

/**
   \brief initializes a TrackElement (sets the default values)
*/
TrackElement::TrackElement(void)
{
  this->isSavePoint = false;
  this->isFork = false;
  this->isJoined = false;
  this->cond; //!< todo think!!
  this->ID = -1;
  this->length =0;
  this->curveType = BEZIERCURVE;
  this->nodeCount = 0;
  childCount = 0;
  this->curve = NULL;
  this->children = NULL;
}

/** 
    \destroys all alocated memory)
    \todo eventually when deleting a TrackElement you would not like to delete all its preceding TrackElements
*/
TrackElement::~TrackElement(void)
{
  if (this->curve)
    delete this->curve;
  if (this->childCount > 0)
    {
      for (int i=0; i < this->childCount; i++)
	delete this->children[i];
    }
}




/**
   \brief standard constructor

   \todo this constructor is not jet implemented - do it
*/
TrackManager::TrackManager () 
{
  this->setClassName ("TrackManager");

  PRINTF(3)("Initializing the TrackManager\n");
  this->firstTrackElem = NULL;
  this->currentTrackElem = firstTrackElem;
  this->localTime = 0;
  this->maxTime = 0;
  this->trackElemCount = 0;
}


/**
   \brief standard destructor

   \todo this deconstructor is not jet implemented - do it
*/
TrackManager::~TrackManager () 
{
  PRINTF(3)("Destruct TrackManager\n");

  PRINTF(3)("Deleting all the TrackElements\n");
  delete this->firstTrackElem;
  // we do not have a TrackManager anymore
  singletonRef = NULL;
}

TrackManager* TrackManager::singletonRef = NULL;

/**
   \returns The reference on the TrackManager.

   If the TrackManager does not exist, it will be created.
*/
TrackManager* TrackManager::getInstance(void) 
{
  if (singletonRef)
    return singletonRef;
  else
    return singletonRef = new TrackManager();
}

/**
   \brief Searches for a given trackID.
   \param trackID the trackID to search for.
   \returns The TrackElement #trackID if found, NULL otherways.
*/
TrackElement TrackManager::findTrackElementByID(int trackID)
{

}

// INITIALIZE //

/**
   \brief Sets the trackID we are working on.
   \param trackID the trackID we are working on
*/
void TrackManager::workOn(int trackID)
{

}

/**
   \brief Sets the Type of the Curve
   \brief curveType The Type to set
*/
void TrackManager::setType(CurveType curveType)
{

}

/**
   \brief Sets the length of the current path in seconds.
   \param time The length in seconds.
*/

void TrackManager::setLength(float time)
{

}

/**
   \brief adds a point to the current TrackElement
   \param newPoint The point to add.
*/
void TrackManager::addPoint(Vector newPoint)
{

}

/**
   \brief adds save/splitpoint.
   \param newPoint The point to add.
*/
void TrackManager::addHotPoint(Vector newPoint)
{

}

/**
   \brief Sets the last HotPoint into a savePoint.
   
   If no HotPoint was defined the last added Point will be rendered into a savePoint. \n
   If the HotPoint was defined as a fork the Point will \b not be set into a savePoint.
*/
void TrackManager::setSavePoint(void)
{
  
}

/**
   \brief adds some interessting non-linear movments through the level.
   \param count The Count of childrens the current HotPoint will have.

   If no HotPoint was defined the last added Point will be rendered into a fork. \n
   If the HotPoint was defined as a savePoint the Point will \b not be set into a fork.
*/
void TrackManager::fork(int count, ...)
{

}

/**
   \brief adds some interessting non-linear movments through the level.
   \param count The Count of childrens the current HotPoint will have.
   \param trackIDs A Pointer to an Array of ints which will hold the trackID's (the user will have to reserve space for this).

   \see void TrackManager::fork(int count, ...)
*/
void TrackManager::forkV(int count, int* trackIDs)
{

}

/**
   \brief decides under what condition a certain Path will be chosen.
   \param groupID the ID on which to choose the preceding move
   \param cond \todo think about this
*/
void TrackManager::condition(int groupID, PathCondition cond)
{

}

/**
   \brief joins some tracks together again. 
   \param count The count of Paths to join.

   Join will set the localTime to the longest time a Path has to get to this Point. \n
   Join will join all curves to the first curve.
*/
void TrackManager::join(int count, ...)
{

}

/**
   \brief joins some tracks together again. 
   \param count The count of Paths to join.
   \param trackIDs an Array with the trackID's to join

   \see void TrackManager::join(int count, ...)
*/
void TrackManager::joinV(int count, int* trackIDs)
{

}

// RUNTIME //

/**
   \brief calculates the Position for the localTime of the Track.
   \returns the calculated Position
*/
Vector TrackManager::calcPos()
{

}

/**
   \brief calculates the Rotation for the localTime of the Track.
   \returns the calculated Rotation
*/
Vector TrackManager::calcDir()
{

}

/**
   \brief Advances the local-time of the Track around dt
   \param dt The time about which to advance.
*/
void TrackManager::tick(float dt)
{

}

/**
   \brief Jumps to a certain point on the Track.
   \param time The time on the Track to jump to.

   This should be used to Jump backwards on a Track, because moving forward means to change between the Path. (it then tries to choose the default.)
   Max is trackLengthMax.
*/
void TrackManager::jumpTo(float time)
{

}

/**
   \brief a Function that decides which Path we should follow.
   \param graphID The Path to choose.
   
*/
void TrackManager::choosePath(int graphID)
{

}

